/*
                                DSP View Widget
 */
#ifndef DSPVIEW_H
#define DSPVIEW_H

#include <sys/types.h>
#include <gtk/gtk.h>

#ifndef gbool
# define gbool  gboolean
#endif


/*
 *	Display styles:
 */
#define DSPVIEW_DISPLAY_STYLE_WAVE	0	/* Solid Volume waves. */
#define DSPVIEW_DISPLAY_STYLE_SGRAPH	1	/* Sine (plotted) graph. */
#define DSPVIEW_DISPLAY_STYLE_ANALYZER	2	/* Horizontal blocks. */
#define DSPVIEW_DISPLAY_STYLE_LIGHTENING	3
#define DSPVIEW_DISPLAY_STYLE_PLASMA	4
#define DSPVIEW_DISPLAY_STYLE_NOVA	5

/*
 *	Number of analyzer blocks to display:
 */
#define DSPVIEW_TOTAL_ANALYZER_BLOCKS	20


/*
 *	DSP merge mode:
 */
#define DSPVIEW_MERGE_MODE_SET		0
#define DSPVIEW_MERGE_MODE_SET_ALPHA	1	/* Transparent by alpha. */
#define DSPVIEW_MERGE_MODE_ADD		2
#define DSPVIEW_MERGE_MODE_SUBTRACT	3
#define DSPVIEW_MERGE_MODE_BLEND	4
#define DSPVIEW_MERGE_MODE_MULTIPLY	5

/*
 *	Default DSP color (0xAABBGGRR):
 */
#define DSPVIEW_DEF_DSP_COLOR	0xffffa0c4

/*
 *	Default fade speed coefficient:
 */
#define DSPVIEW_DEF_FADE_COEFF	0.3

/*
 *	DSP View structure.
 */
typedef struct {

        gbool initialized;
        gbool map_state;
	GdkVisibilityState visibility;
	GtkWidget	*toplevel,
			*view_da;

	gint	channels;	/* 1, 2, or 4. */
	gint	display_style;	/* One of DSPVIEW_DISPLAY_STYLE_*. */
	gint	merge_mode;	/* One of DSPVIEW_MERGE_MODE_*, the
				 * merge_mode does not have affect on
				 * all display_styles, certain
				 * display_styles impose their own merge
				 * modes.
				 */
	guint32	dsp_color;	/* 0xAABBGGRR. */
	gfloat	fade_coeff;	/* Fade speed coefficient (0.0 to 1.0].
				 * 1.0 for strongest and fastest fade.
				 * The fade speed coefficient has
				 * different affects depending on the
				 * display_style and has no affect on
				 * certain display styles.
				 */

	/* view_da "back" buffer 32 bit 0xAABBGGRR format. */
	guint8	*buf;
	gint	width, height, bpl, bpp;

	/* DSP "history" buffer (not used with some display styles), 
	 * allocation size and format must match the view_da "back"
	 * buffer buf.
	 */
	guint8  *dsp_buf;

	/* Background image buffer (shared). */
	const guint8	*bg_buf;
        gint		bg_width, bg_height,
			bg_bpl, bg_bpp;

	/* Analyzer history block positions, used only if display_style
	 * is set to DSPVIEW_DISPLAY_STYLE_ANALYZER.
	 */
	guint8 analyzer_history_buf[DSPVIEW_TOTAL_ANALYZER_BLOCKS];

} dspview_struct;

#define DSPVIEW(p)	((dspview_struct *)(p))
#define DSPVIEW_TOPLEVEL(p)	(((p) != NULL) ? \
 (DSPVIEW(p)->toplevel) : NULL)
#define DSPVIEW_VIEW(p)		(((p) != NULL) ? \
 (DSPVIEW(p)->view_da) : NULL)


extern dspview_struct *DSPViewNew(
	GtkWidget *parent, gint width, gint height
);
extern void DSPViewSetBG(
	dspview_struct *dv,
	const guint8 *buf,
	gint width, gint height, gint bpl, gint bpp
);
extern void DSPViewSetChannels(
        dspview_struct *dv, gint channels
);
extern void DSPViewSetDisplayStyle(
	dspview_struct *dv, gint style
);
extern void DSPViewSetMergeMode(
        dspview_struct *dv, gint mode
);
extern void DSPViewSetDSPColor(
	dspview_struct *dv, guint32 c
);
extern void DSPViewSetFadeCoeff(
        dspview_struct *dv, gfloat v
);
extern void DSPViewDraw(
	dspview_struct *dv,
	const guint8 *buf, gint len
);
extern void DSPViewMap(dspview_struct *dv);
extern void DSPViewUnmap(dspview_struct *dv);
extern void DSPViewDelete(dspview_struct *dv);




#endif	/* DSPVIEW_H */
